#version 330

in vec4 vs_fs_color; // interpolated between vertices
in vec3 vs_fs_normal; // surface normal, interpolated between vertices
in vec4 vs_fs_position;

out vec4 FragColor; // color result for this fragment

void main()
{
	
	vec3 Ambient; // sets lighting level, same across many vertices
	vec3 LightColor;
	vec3 LightPosition; // location of the light, eye space
	vec3 ReflectionVector; // surface orientation for shiniest spots
	vec3 EyeVector;
	float Shininess; // exponent for sharping highlights
	float Strength; // extra factor to adjust shininess
	vec3 EyeDirection;
	
	Ambient = vec3(0.4, 0.4, 0.4);
	LightColor = vec3(0.3, 0.0, 0.0);
	LightPosition = vec3(0.0, 0.0, 1.0);
	Shininess = 0.7;
	Strength = 1.0;
	EyeDirection = vec3(0.0, 0.0, 0.0);
	
	EyeVector = EyeDirection - vec3(vs_fs_position);;
	EyeVector = normalize(EyeVector);
	
	// find the direction and distance of the light,
	// which changes fragment to fragment for a local light
	vec3 lightDirection = LightPosition - vec3(vs_fs_position);
	float lightDistance = length(lightDirection);
	
	// normalize the light direction vector, so
	// that a dot products give cosines
	lightDirection = lightDirection / lightDistance;
	
	// Reflection Vector
	ReflectionVector = 2 * dot(lightDirection, vs_fs_normal) * vs_fs_normal - lightDirection;
	ReflectionVector = normalize(ReflectionVector);
	
	// compute cosine of the directions, using dot products,
	// to see how much light would be reflected
	float diffuse = max(0.0, dot(vs_fs_normal, lightDirection));
	float specular = max(0.0, dot(ReflectionVector, EyeVector));
	
	// surfaces facing away from the light (negative dot products)
	// won't be lit by the directional light
	if (diffuse == 0.0)
		specular = 0.0;
	else
		specular = pow(specular, Shininess) * Strength; // sharpen the highlight
	
	vec3 scatteredLight = Ambient + LightColor * diffuse;
	vec3 reflectedLight = LightColor * specular;
	
	// don't modulate the underlying color with reflected light,
	// only with scattered light
	vec3 rgb = min(vs_fs_color.rgb * scatteredLight + reflectedLight, vec3(1.0));

	FragColor = vec4(rgb, vs_fs_color.a);
	
}
